#!/bin/ksh
# *****************************************************************************
#  Patch #3 for Service Profiler v1.3
# *****************************************************************************
#  Wrightia, S.A. (c) 2012
#
#  Purpose:
#     1. Replace all native libraries
#     2. Replace all server and static JAR libraries
#
#  Input parameters:
#     $1          : IS_DIR  (The Integration Server installation directory)
#
#  Results:
#     1.  Applies Patch #3 for Service Profiler v1.3.
#
#  Exit codes:
#     0  - OK
#
# *****************************************************************************

log() {
   echo "[$1] [`date +%C%y-%m-%d\ %H:%M:%S`] $2" >> updates.log
}

logError() {
   log "ERROR" "$1"
}

logWarning() {
   log "WARNING" "$1"
}

logInfo() {
   log "INFO" "$1"
}

#--< MAIN	>------------------------------------------------------------------
if [ -d ../error ]; then
   if [ ! -z "`ls ../error/ | sort | grep .upd.sh`" ] ; then
      logWarning "Previous updates with error were found: '$0 $*' skipped."
      exit 100
   fi
else
   mkdir ../error
fi

xISHome=$1
xTargetHome=${xISHome}/serviceprofiler
if [ -d "$xTargetHome" ] ; then
   # replace previous installation on server folder
   rm -rf "$xTargetHome"
   cp -r ../../serviceprofiler "${xISHome}/"
   xEL=$?
   if [ $xEL -eq 0 ]; then
      logInfo "All files copied successfully."
   else
      logError "Error '$xEL' copying files."
      exit $xEL
   fi
else
   logWarning "Directory '${xISHome}/serviceprofiler' was not found: '$0 $*' not applied."
fi

