/*------------------------------------------------------------------------------
   Global note: do not change the order that the functions and static vars are
                declared. Its could disrupt dependencies.
 -----------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
   Extends to already existing classes
 -----------------------------------------------------------------------------*/
String.prototype.trim = function() { return this.replace(/^\s+|\s+$/g, ""); };

String.replicate = function(character, times)
   /****************************************************************************
   *  Create a new String filled with the given character, the given times.
   ***************************************************************************/{
      var xResult = "";

      for(var xIdx = 0; xIdx < times; xIdx++)
         xResult += character;

      return xResult;
   }; // END replicate(char, int)

String.prototype.padL = function(length, paddingChar)
   /****************************************************************************
   *  Pads a String with characters to the left up to a given length.
   ***************************************************************************/{
      return String.replicate(paddingChar, length - this.length) + this;
   }; // END padL(String, int, char)


/*------------------------------------------------------------------------------
   CLASSES
 -----------------------------------------------------------------------------*/

function TableSorter()
/*******************************************************************************
* Sort rows of a HTML TABLE
*     by Antnio Abreu
******************************************************************************/{
} // END class TableSorter


function DateFormat(mask)
/*******************************************************************************
* Controls a date string value according to a mask.
*
* Only recognizes:
*  - 'dd' as placement of the day value.
*  - 'MM' as placement of the month value.
*  - 'yy' as placement of the 2-digit year value.
*  - 'yyyy' as placement of the 4-digit year value.
*
*  Default mask is 'dd-MM-yyyy'
*
*     by Antnio Abreu
*******************************************************************************/{
   this.dayOffset    = 0;
   this.dayLength    = 2;
   this.monthOffset  = 3;
   this.monthLength  = 2;
   this.yearOffset   = 6;
   this.yearLength   = 4;
   this.init(mask);
}  // END class DateFormat

DateFormat.prototype.init = function(mask)
/*****************************************************************************/{
   if (mask != null)
      {
      this.dayOffset    = mask.indexOf("dd");
      this.monthOffset  = mask.indexOf("MM");
      this.yearOffset   = mask.indexOf("yyyy");
      if (this.yearOffset == -1)
         {
         this.yearOffset   = mask.indexOf("yy");
         this.yearLength   = 2;
         }
      }
} // END init(string)

DateFormat.prototype.parse = function(value)
/*****************************************************************************/{
   var xResult = value.substring(this.yearOffset, this.yearOffset + this.yearLength);

   if (this.yearLength == 2)
      if (parseInt(xResult) < 50)
         xResult = '20' + xResult;
      else
         xResult = '19' + xResult;

   return xResult.concat(value.substring(this.monthOffset, this.monthOffset + this.monthLength)).concat(value.substring(this.dayOffset, this.dayOffset + this.dayLength));
} // END parse(string)


function NumberFormat(decimalPoint)
/*******************************************************************************
* Controls a numeric string value according to a mask.
* Allows parsing (still to a string) of numeric values, with or without
* digit grouping, and any currency value.
*
* Parameters:
*  - decimalPoint
*     The char, or string of chars, that is the decimal-point on the value.
*
*     by Antnio Abreu
*******************************************************************************/{
   this.decimalPoint = decimalPoint;
   this.preParserRE  = new RegExp("[^\-0-9" + decimalPoint + "]", "g");
}  // END class NumberFormat

NumberFormat.prototype.parse = function(value)
/*****************************************************************************/{
   var xResult = value.replace(this.preParserRE, "");
   return xResult.replace(this.decimalPoint, ".");
} // END parse(string)


/*------------------------------------------------------------------------------
   CLASS STATIC ELEMENTS
 -----------------------------------------------------------------------------*/

// static variables
TableSorter.imagesFolder       = "tablesorter/images";
TableSorter.ArrowUpImageFile   = "arrow-up.gif";
TableSorter.ArrowDownImageFile = "arrow-down.gif";
// internal use
TableSorter.workingColumnIndex = -1;
TableSorter.sortOption         = null;
// informational
TableSorter.versionId          = "1.0.3";
TableSorter.versionDate        = "12-AUG-2009";
TableSorter.author             = "Antnio Abreu";

// static methods
TableSorter.init = function()
/*******************************************************************************
* Find all tables with class "sortable" and prepare them for sorting
******************************************************************************/{

   if (!document.getElementsByTagName)
      return;                    // not supported by browser

   var xTables = document.getElementsByTagName("table");

   for(var xIdx = 0; xIdx < xTables.length; xIdx++)
      {
      var xThisTable = xTables[xIdx];

      if (xThisTable.getAttribute("sortable") == "true")
          TableSorter.prepare(xThisTable);
      }
} // END TableSorter.init()


TableSorter.prepare = function(table)
/*******************************************************************************
* Find all cells in the table header with attribute "sortable" and prepare
* them for sorting.
******************************************************************************/{
   var xHeaderRow, xHeaderRowCell;

   for(var xRowIdx = 0; xRowIdx < table.tHead.rows.length; xRowIdx++)
      {
      xHeaderRow = table.tHead.rows[xRowIdx];

      for(var xCellIdx = 0; xCellIdx < xHeaderRow.cells.length; xCellIdx++)
         {
         xHeaderRowCell  = xHeaderRow.cells[xCellIdx];
         var xIsSortable = xHeaderRowCell.getAttribute("sortable");

         if (xIsSortable)
            {
            var xCellText     = xHeaderRowCell.innerHTML;
            var xSortType     = xHeaderRowCell.getAttribute("sortType");
            var xSortOrder    = xHeaderRowCell.getAttribute("sortOrder");
            var xSortOption   = xHeaderRowCell.getAttribute("sortOption");
            var xHTML         = '<a href="#" ' + TableSorter.getAnchorStyleAttribute(xHeaderRowCell);

            // transfer sort TD attributes to clickable-link for future evaluations
            if (xSortType != null)
               xHTML += ' sortType="' + xSortType + '"';

            if (xSortOrder != null)
               xHTML += ' sortOrder="' + xSortOrder + '"';

            if (xSortOption != null)
               xHTML += ' sortOption="' + xSortOption + '"';

            xHTML += ' onclick="TableSorter.execute(this);">'
                   + xCellText
                   + '<span class="arrow"></span></a>';  // add placeholder (do not tamper with!)

            xHeaderRowCell.innerHTML = xHTML;
            }
         }
      }
} // END prepare(table)

TableSorter.execute = function(link)
/*****************************************************************************/{
   // get the <span> element
   var xSpanElement;

   for(var xIdx = 0; xIdx < link.childNodes.length; xIdx++)
      if (link.childNodes[xIdx].tagName && link.childNodes[xIdx].tagName.toLowerCase() == 'span')
         xSpanElement = link.childNodes[xIdx];

   var xSpanElementText  = TableSorter.getInnerText(xSpanElement);
   var xTableDataElement = link.parentNode;
   var xColumnIndex      = xTableDataElement.cellIndex;
   var xTableElement     = TableSorter.getParent(xTableDataElement, 'TABLE');
   var xTableBody        = xTableElement.tBodies[0];      // default table TBODY

   // Overcome Opera 9 bug:
   //    - Any TBODY of any inner table will be listed as .tBodies of the enclosing table... the jackhass!
   if (navigator.appName == "Opera")
      for(var xIdx = 0; xIdx < xTableElement.tBodies.length; xIdx++)
         {
         var xParentTable = xTableElement.tBodies[xIdx].rows[0].parentNode.parentNode;
         if (xParentTable && xParentTable.getAttribute("sortable"))
            {
            xTableBody = xTableElement.tBodies[xIdx];
            break;
            }
         }

   if (xTableBody.rows.length == 0)
      return false;

   // determine the sort function to use
   var xCompareFunction  = TableSorter.compareIgnoreCase;
   var xSortType         = link.getAttribute("sortType");
   var xDefaultSortOrder = link.getAttribute("sortOrder");
   var xSortOption       = link.getAttribute("sortOption");

   if (xSortType == "time")
      xCompareFunction = TableSorter.compareTime;
   else if (xSortType == "date")
      xCompareFunction = TableSorter.compareDate;
   else if (xSortType == "number")
      xCompareFunction = TableSorter.compareNumeric;
   else if (xSortType == "text")
      xCompareFunction = TableSorter.compareIgnoreCase;
   else if (xSortType == "currency")
      xCompareFunction = TableSorter.compareNumeric;
   else if (xSortType == "image")
      xCompareFunction = TableSorter.compareImage;
   else
      {  // Work out a type for the column
      var xDemoCell = xTableBody.rows[0].cells[xColumnIndex];
      var xItem     = TableSorter.getInnerText(xDemoCell);

      if      (xItem.match(/^\d\d[\/-]\d\d[\/-]\d\d\d\d$/))  xCompareFunction = TableSorter.compareDate;
      else if (xItem.match(/^\d\d[\/-]\d\d[\/-]\d\d$/))      xCompareFunction = TableSorter.compareDate;
      else if (xItem.match(/^[$]/))                        xCompareFunction = TableSorter.compareNumeric;
      else if (xItem.match(/^[\d\.]+$/))                     xCompareFunction = TableSorter.compareNumeric;
      else if (xDemoCell.hasChildNodes())
         {
         var xIMGList = xDemoCell.getElementsByTagName("img");

         if (xIMGList && xIMGList.length == 1)  // when the table cell contains one, and only one, image
            xCompareFunction = TableSorter.compareImage;
         }
      }

   if (xCompareFunction == TableSorter.compareNumeric && xSortOption == null)
      xSortOption = ".";               // default decimal point character
   else if (xCompareFunction == TableSorter.compareDate && xSortOption == null)
      xSortOption = "dd-MM-yyyy";      // default date value mask

   TableSorter.workingColumnIndex = xColumnIndex;
   if      (xCompareFunction == TableSorter.compareNumeric)
      TableSorter.sortOption = new NumberFormat(xSortOption);
   else if (xCompareFunction == TableSorter.compareDate)
      TableSorter.sortOption = new DateFormat(xSortOption);
   else
      TableSorter.sortOption = null;

   var xNewRowsArray = new Array();

   for(var xIdx = 0; xIdx < xTableBody.rows.length; xIdx++)
     xNewRowsArray[xIdx] = xTableBody.rows[xIdx];

   xNewRowsArray.sort(xCompareFunction);

   var xSortOrder = xSpanElement.getAttribute("sortOrder");
   if (xSortOrder == null || xSortOrder == "")
      xSortOrder = xDefaultSortOrder;

   var xArrowImageHTML = '<img style="border-style: none;" src="' + TableSorter.imagesFolder + '/';
   if (xSortOrder == 'desc')
      {
      xNewRowsArray.reverse();
      xArrowImageHTML += TableSorter.ArrowDownImageFile + '">';
      xSpanElement.setAttribute('sortOrder', 'asc');
      }
   else
      {
      xArrowImageHTML += TableSorter.ArrowUpImageFile + '">';
      xSpanElement.setAttribute('sortOrder', 'desc');
      }

   // collect the classes of the table
   var xClasses = new Array(xNewRowsArray.length);

   for(var xRowIdx = 0; xRowIdx < xClasses.length; xRowIdx++)
      xClasses[xRowIdx] = new Array(xNewRowsArray[0].cells.length);

   for(var xRowIdx = 0; xRowIdx < xTableBody.rows.length; xRowIdx++)
      for(var xCellIdx = 0; xCellIdx < xTableBody.rows[xRowIdx].cells.length; xCellIdx++)
         xClasses[xRowIdx][xCellIdx] =  xTableBody.rows[xRowIdx].cells[xCellIdx].className;

   // append child rows that already exist to the tbody,
   // so it moves them rather than creating new ones
   for(var xRowIdx = 0; xRowIdx < xNewRowsArray.length; xRowIdx++)
      {
      // update table cell's classes in their original positions
      for(var xCellIdx = 0; xCellIdx < xNewRowsArray[xRowIdx].cells.length; xCellIdx++)
         xNewRowsArray[xRowIdx].cells[xCellIdx].className = xClasses[xRowIdx][xCellIdx];

      xTableBody.appendChild(xNewRowsArray[xRowIdx]);
      }

   xSpanElement.innerHTML = xArrowImageHTML;

   {  // Delete arrows showing on columns other than the just sorted
      var xAllSpanElements = document.getElementsByTagName('span');

      for(var xIdx = 0; xIdx < xAllSpanElements.length; xIdx++)
         if (xAllSpanElements[xIdx].className == 'arrow')
            if (xAllSpanElements[xIdx] != xSpanElement)  // not the working SPAN element
               if (TableSorter.getParent(xAllSpanElements[xIdx], 'table') == TableSorter.getParent(link, 'table')) // same as the 'working' table?
                  {
                  xAllSpanElements[xIdx].innerHTML = '';
                  xAllSpanElements[xIdx].setAttribute('sortOrder', '');
                  }
   }

   return false;
} // END TableSorter.execute(link)

TableSorter.addEventTo = function(element, eventType, eventFunction, useCapture)
/*******************************************************************************
* Add an event handler to an Element.
* Cross-browser event handling for IE5+, NS6 and Mozilla.
*     By Scott Andrew
******************************************************************************/{
   if (element.addEventListener)
      {
      element.addEventListener(eventType, eventFunction, useCapture);
      return true;
      }
   else if (element.attachEvent)
      return element.attachEvent("on" + eventType, eventFunction);
   else
      alert("Event handler " + eventFunction + "could not be added.");
} // END TableSorter.addEvent(element, eventType, eventFunction, useCapture)


/*------------------------------------------------------------------------------
   SORT FUNCTIONS
 -----------------------------------------------------------------------------*/

TableSorter.compareDate = function(a, b)
/*****************************************************************************/{
   // y2k notes: two digit years less than 50 are treated as 20XX, greater than 50 are treated as 19XX
   var xTmpA = TableSorter.getInnerText(a.cells[TableSorter.workingColumnIndex]);
   var xTmpB = TableSorter.getInnerText(b.cells[TableSorter.workingColumnIndex]);

   xTmpA = TableSorter.parseDate(xTmpA);
   xTmpB = TableSorter.parseDate(xTmpB);

   return TableSorter.compare(xTmpA, xTmpB);
} // END TableSorter.compareDate(td, td)

TableSorter.compareNumeric = function(a, b)
/*****************************************************************************/{
   var xTmpA = TableSorter.parseNumber(TableSorter.getInnerText(a.cells[TableSorter.workingColumnIndex]));
   var xTmpB = TableSorter.parseNumber(TableSorter.getInnerText(b.cells[TableSorter.workingColumnIndex]));

   return xTmpA - xTmpB;
} // TableSorter.compareNumeric(td, td)

TableSorter.compareIgnoreCase = function(a, b)
/*****************************************************************************/{
   var xTmpA = TableSorter.getInnerText(a.cells[TableSorter.workingColumnIndex]).toLowerCase();
   var xTmpB = TableSorter.getInnerText(b.cells[TableSorter.workingColumnIndex]).toLowerCase();

   return TableSorter.compare(xTmpA, xTmpB);
} // END TableSorter.compareIgnoreCase(td, td)

TableSorter.compareImage = function(a, b)
/*****************************************************************************/{
   var xTmpA = a.cells[TableSorter.workingColumnIndex].getElementsByTagName("img")[0];
   var xTmpB = b.cells[TableSorter.workingColumnIndex].getElementsByTagName("img")[0];

   xTmpA = xTmpA.getAttribute('src');
   xTmpB = xTmpB.getAttribute('src');

   return TableSorter.compare(xTmpA, xTmpB);
} // END TableSorter.compareImage(td, td)

TableSorter.compareTime = function(a, b)
/*****************************************************************************/{
   var xTmpA = TableSorter.getInnerText(a.cells[TableSorter.workingColumnIndex]);
   var xTmpB = TableSorter.getInnerText(b.cells[TableSorter.workingColumnIndex]);

   xTmpA = xTmpA.padL(16, " ");
   xTmpB = xTmpB.padL(16, " ");

   return TableSorter.compare(xTmpA, xTmpB);
} // END TableSorter.compareTime(td, td)

TableSorter.compareTd = function(a, b)
/*****************************************************************************/{
   var xTmpA = TableSorter.getInnerText(a.cells[TableSorter.workingColumnIndex]);
   var xTmpB = TableSorter.getInnerText(b.cells[TableSorter.workingColumnIndex]);

   return TableSorter.compareTd(xTmpA, xTmpB);
}  // END TableSorter.compareTd(td, td)

TableSorter.compare = function(arg1, arg2)
/*****************************************************************************/{
   if (arg1 == arg2)
      return 0;
   else if (arg1 < arg2)
      return -1;
   else
      return 1;
}  // TableSorter.compare(String, String)


/*------------------------------------------------------------------------------
   UTILITY
 -----------------------------------------------------------------------------*/
TableSorter.getInnerText = function(element)
/*****************************************************************************/{
   var xResult = "";

   if (typeof element == "string")
      return element.trim();
   else if (typeof element == "undefined")
      return element;
   else if (element.innerText)
      xResult = element.innerText;   //Not needed, but it is faster
   else
      {
      var xChildNodes      = element.childNodes;
      var xChildNodesCount = xChildNodes.length;

      xResult = "";
      for(var xIdx = 0; xIdx < xChildNodesCount; xIdx++)
         {
         switch (xChildNodes[xIdx].nodeType)
            {
            case 1:     // ELEMENT_NODE
               xResult += TableSorter.getInnerText(xChildNodes[xIdx]);
               break;

            case 3:     // TEXT_NODE
               xResult += xChildNodes[xIdx].nodeValue;
               break;
            }
         }
      }

   return xResult.trim();
}  // END TableSorter.getInnerText(element)

TableSorter.getAnchorStyleAttribute = function(rowCell)
/*******************************************************************************
*  Returns the 'style' attribute for the sort Anchor/Link.
*  Deals with different DOM implementations, specifically for
*  Gecko (Netscape et all) and IE.
*  It tries to not change the color decoration of the text.
******************************************************************************/{
   var xResult    = 'style="text-decoration: none; color: ';
   var xFontColor = '#FFFFFF';       // set default color

   if (rowCell != null)
      {
      if (rowCell.currentStyle != null)        // IE
         xFontColor  = rowCell.currentStyle.color;
      else if (document.defaultView != null)
         xFontColor  = document.defaultView.getComputedStyle(rowCell, '').getPropertyValue('color');
      }

   return xResult + xFontColor + ';"';
}  // END TableSorter.getAnchorStyleAttribute(rowCell)

TableSorter.getParent = function(element, tagName)
/*****************************************************************************/{
   if (element == null)
      return null;
   else if (element.nodeType == 1 && element.tagName.toLowerCase() == tagName.toLowerCase()) // Gecko bug, supposed to be uppercase
      return element;
   else
      return TableSorter.getParent(element.parentNode, tagName);
}  // TableSorter.getParent(element, String)

TableSorter.parseDate = function(dateStr)
/*******************************************************************************
* Return the given text dateStr formatted as a 'yyyymmdd' formated string.
*  by Antnio Abreu
******************************************************************************/{
   return TableSorter.sortOption.parse(dateStr);
}// END TableSorter.parseDate(String)

TableSorter.parseNumber = function(value)
/*******************************************************************************
* Parse a string with a numeric value.
*  by Antnio Abreu
******************************************************************************/{
   var xResult;
   var xPreParsedValue = value;

   if (TableSorter.sortOption != null)
      xPreParsedValue = TableSorter.sortOption.parse(value);

   xResult = parseFloat(xPreParsedValue);
   if (isNaN(xResult))
      xResult = -1;

   return xResult;
}// END TableSorter.parseNumber(String)


/*------------------------------------------------------------------------------
   Static initialization
 -----------------------------------------------------------------------------*/
TableSorter.addEventTo(window, "load", TableSorter.init);

